-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp, SQLskills.com
-- All Rights Reserved.
-- 
-- For more scripts, sample code and additional training resources,
-- go to http://www.SQLskills.com
--	Immerse yourself in SQL Server
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


/* ****************************************************************
This script will create another copy of the Pubs database named
PubsTest. PubsTest will be created in the default location of SQL
SQL Server on drive c:\. Change C:\Program Files\M... if you did NOT
install SQL Server in the default directory/path make sure you
MODIFY this script before running it. In addition to copying Pubs in 
the first restore - this script will add 3 files TOTALLING 18 MB of 
additional space. In all this demo requires roughly 30 MB of space.

Also, this script is NOT intended to be executed in one run.

Work your way through this script reading and reviewing the comments
for better understanding.

As you execute portions of this script, you will modify records in a
secondary PubsTest database and create different types of backups. Once
backed up you will use queries to determine what is the likely order 
of file backups and log backups needed to restore. There are a couple 
of questions that should help you test yourself!

Have fun,
kt
*/

SET NOCOUNT ON
go

USE pubs
go

BACKUP DATABASE [pubs] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat' WITH INIT
go

RESTORE DATABASE [PubsTest] 
	FROM  DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat' 
	WITH  FILE = 1,  NOUNLOAD ,  STATS = 10,  RECOVERY ,  
MOVE N'pubs_log' TO N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\DATA\PubsTestLog.ldf',  
MOVE N'pubs' TO N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\data\PubsTest.mdf'
	, REPLACE
go

ALTER DATABASE [PubsTest] 
	SET RECOVERY FULL
go

USE [PubsTest] 
go

-- Increase the size of the Transaction Log for later activity
ALTER DATABASE [PubsTest] 
MODIFY FILE
		(NAME = N'pubs_log', 
		SIZE = 5, FILEGROWTH = 10%)
go

-- Add Additional Files and a FileGroup.
ALTER DATABASE [PubsTest] ADD FILEGROUP [TestFG] 
go

ALTER DATABASE [PubsTest] 
	ADD FILE
		(NAME = N'PubsTestFile1', 
		FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\data\PubsTestFile1.NDF' , 
		SIZE = 6, FILEGROWTH = 10%) 
		TO FILEGROUP [TestFG]
go
ALTER DATABASE [PubsTest] 
	ADD FILE
		(NAME = N'PubsTestFile2', 
		FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\data\PubsTestFile2.NDF' , 
		SIZE = 6, FILEGROWTH = 10%) 
		TO FILEGROUP [TestFG]
go
ALTER DATABASE [PubsTest] 
	ADD FILE
		(NAME = N'PubsTestFile3', 
		FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\data\PubsTestFile3.NDF' , 
		SIZE = 6, FILEGROWTH = 10%) 
		TO FILEGROUP [TestFG]
go
USE PubsTest
go
sp_helpfile -- To Review the current number of and size of the database files.
go
ALTER DATABASE PubsTest
	MODIFY FILEGROUP [TestFG] DEFAULT
-- This allows you to place objects on the Filegroup WITHOUT specifying it every time!
go

CREATE TABLE dbo.DemoTable
(
	col1	int	 identity(100,10),
	col2	datetime default current_timestamp,
	col3	datetime default getdate(),
	col4	char(30) default suser_name(),
	col5	char(30) default user_name(),
	col6	char(80) default 'Now is the time for all good men to come to the aid of their country.'
) -- ON [TestFG] -- this is NOT required if TestFG is the DEFAULT FileGroup.
go

SELECT count(*) FROM dbo.DemoTable	-- 0 Rows but the table does exist.
go

DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

SELECT count(*) FROM dbo.DemoTable	-- 2000 Rows
go

DBCC SQLPERF(logspace)
-- PubsTest should show about 30% although if you have modified any of the settings above
-- then this may be different.
go

-- FYI - the following two commands should be commented out... They are here for you to
-- review if you're interested in FILEGROUP syntax examples.

-- ALTER DATABASE JunkDB
-- 		MODIFY FILEGROUP [fg] READONLY
-- go
-- ALTER DATABASE JunkDB
-- 		MODIFY FILEGROUP [fg] READWRITE
-- go


-- Now that the database is ready we can begin different backups. A regular strategy for a database
-- this size would be FULL and TLOG however larger databases would add periodic DIFFs. VLDBs would
-- break it down into FILE/FILEGROUP and TLOG Backups and even larger databases would use 
-- FILE/FG DIFFs.

-- So - to demostrate the point of how to use File/FG backups - and differentials. We will create
-- a series of backups with the PubsTest database.

-- First, Create FILE BACKUPS to use as a starting point for recovery. These could be staggered over
-- a few days (when the files themselves are so large that you are unable to back them up at the same
-- time).

sp_helpfile -- to review File/Filegroup Names
go
-- FileName			#	Physical Location													Filegroup	Size	Max Size	Growth	Usage
-- pubs				1	C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\data\PubsTest.mdf 		PRIMARY		1600 KB	Unlimited	10%		data only	
-- pubs_log    		2	C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\DATA\PubsTestLog.ldf    NULL		5120 KB	Unlimited	10%		log only
-- PubsTestFile1	3	c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\data\PubsTestFile1.NDF  TestFG		6144 KB	Unlimited	10%		data only	
-- PubsTestFile2	4	c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\data\PubsTestFile2.NDF  TestFG		6144 KB	Unlimited	10%		data only	
-- PubsTestFile3   	5	c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\data\PubsTestFile3.NDF  TestFG		6144 KB	Unlimited	10%		data only	

-- Clear all PubsTest Backup History First!
USE msdb
go

EXEC sp_delete_database_backuphistory 'PubsTest'
go

USE PubsTest
go

BACKUP DATABASE [PubsTest] FILE = 'pubs'
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, File = pubs', INIT
go

-- Change some more data
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the Log
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, Transaction Log', NOINIT
go

-- Change some more data
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the FileGroup OR the 3 Files Individually (the filegroup is easier but
-- may be too large in some environments). The individual file backups are commented
-- out.

BACKUP DATABASE [PubsTest] FILEGROUP = 'TestFG'
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, FileGroup = TestFG', NOINIT
go
-- BACKUP DATABASE [PubsTest] FILE = 'PubsTestFile1'
-- 	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat'
--	WITH NAME = 'PubsTest Backup, File = PubsTestFile1', NOINIT
-- go
-- BACKUP DATABASE [PubsTest] FILE = 'PubsTestFile2'
-- 	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat'
--	WITH NAME = 'PubsTest Backup, File = PubsTestFile2', NOINIT
-- go
-- BACKUP DATABASE [PubsTest] FILE = 'PubsTestFile3'
-- 	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat'
--	WITH NAME = 'PubsTest Backup, File = PubsTestFile3', NOINIT

-- Change some more data
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the Log
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, Transaction Log', NOINIT
go

-- Change some more data
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the FileGroup with Differential
BACKUP DATABASE [PubsTest] FILEGROUP = 'TestFG'
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, FileGroup DIFFERENTIAL = TestFG', DIFFERENTIAL, NOINIT
go

-- Change some more data
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Create an object that modifies the Primary Filegroup
CREATE PROCEDURE dbo.GetDataFromDemoTable
AS
SELECT * FROM dbo.DemoTable
WHERE Col1 % 19 = 0
go

-- Backup the Log
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, Transaction Log', NOINIT
go

-- Change some more data
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

BACKUP DATABASE [PubsTest] FILE = 'pubs'
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, File = pubs', NOINIT
go

-- Change some more data
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the Log
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, Transaction Log', NOINIT
go

-- Change some more data
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the FileGroup with Differential
BACKUP DATABASE [PubsTest] FILEGROUP = 'TestFG'
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, FileGroup DIFFERENTIAL = TestFG'
	, DIFFERENTIAL, NOINIT
go

-- Change some more data
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the Log
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, Transaction Log', NOINIT
go

-- Change some more data
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Review the backup history in MSDB
SELECT Backup_Start_Date, 
	[Name],
	[Description],
	First_LSN, 
	Last_LSN, 
	Backup_Finish_Date, 
	* -- OR Just *
FROM msdb.dbo.backupset AS s
    JOIN msdb.dbo.backupmediafamily AS m
        ON s.media_set_id = m.media_set_id
WHERE database_name = 'PubsTest'
ORDER BY 1 ASC

-- Now you have a complete set of file/filegroup/filegroup differentials and tlogs...
-- Using the information from msdb figure out what the correct options are for these scenarios:

-- 1) How can you recover up to the minute of the failure if you were to lose ONLY TestPubsFile2?

-- 2) What do you need to do to completely recover the entire database in another location?

-- See "Restore File Backups - Answers.sql" for the answers and syntax!

-- Have fun!!!
-- Kimberly